// Low-level geocoding interface - wraps specific API calls and their
// return values.
mapbox.geocoder = function(_) {
    var geocoder = {};

    geocoder.getURL = function(_) {
        return url;
    };

    geocoder.setURL = function(_) {
        url = _;
        return geocoder;
    };

    geocoder.setID = function(_) {
        return geocoder.setURL(mapbox.base() + _ + '/geocode/{query}.json');
    };

    geocoder.setTileJSON = function(_) {
        return geocoder.setID(_.id || '');
    };

    geocoder.queryUrl = function(_) {
        return L.Util.template(this.getURL(), { query: _ });
    };

    geocoder.query = function(_, callback) {
        mapbox.request(this.queryUrl(_), function(err, json) {
            if (json && json.results && json.results.length) {
                var res = {
                    results: json.results,
                    latlng: [json.results[0][0].lat, json.results[0][0].lon]
                };
                if (json.results[0][0].bounds !== undefined) {
                    res.bounds = json.results[0][0].bounds;
                    res.lbounds = mapbox.lbounds(res.bounds);
                }
                callback(null, res);
            } else callback(err);
        });

        return geocoder;
    };

    // a reverse geocode:
    //
    //  geocoder.reverseQuery([80, 20])
    geocoder.reverseQuery = function(_, callback) {
        var q = '';

        function norm(x) {
            if (x.lat !== undefined && x.lng !== undefined) return x.lng + ',' + x.lat;
            else if (x.lat !== undefined && x.lon !== undefined) return x.lon + ',' + x.lat;
            else return x[0] + ',' + x[1];
        }

        if (_.length && _[0].length) {
            for (var i = 0, pts = []; i < _.length; i++) pts.push(norm(_[i]));
            q = pts.join(';');
        } else q = norm(_);

        mapbox.request(this.queryUrl(q), function(err, json) {
            callback(err, json);
        });

        return geocoder;
    };

    if (typeof _ === 'string') mapbox.idUrl(_, geocoder);
    else if (typeof _ === 'object') geocoder.setTileJSON(_);

    return geocoder;
};
